import 'package:adeptarch/adeptarch.dart';
import 'package:adeptarch/src/api/graphql_service.dart';
import 'package:adeptarch/src/api/query_response.dart';
import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';
import 'package:http/http.dart' as http;
import 'package:adeptarch/src/api/graphql_api.dart' as GraphQLAPI;
import 'dart:async';

/// Created by krrigan on 04/28/20.

abstract class GraphQLAPIService<I extends GraphQLService, DataType, ErrorType>
{
  I service;
  DataType dataType;
  ErrorType errorType;
  GraphQLAPI.GraphQLAPI graphQLAPI;
  QueryResponseConverter<DataType> queryResponseConverter;

  GraphQLAPIService(this.service, {
    this.dataType,
    this.errorType,
    this.queryResponseConverter,
    this.graphQLAPI}) {
    graphQLAPI ??= GraphQLAPI.graphQLAPI;
    service?.graphQLAPI ??= graphQLAPI;
    queryResponseConverter ??= QueryResponseConverter<DataType>(data: dataType);
  }

  dynamic getAuthorization();

  updateHttpClient(http.Client httpClient){
    if(httpClient != null) graphQLAPI.init(httpClient: httpClient);
  }

  Future<QueryResult> getSaveResponse(Future<QueryResult> futureResponse) async{
    if(futureResponse == null) return null;
    QueryResult response;
    try{
        response = await futureResponse;
    } catch(e){
        if(e is QueryResult) response = e; else throw e;
    }
    return response;
  }

  Future<QueryResponse<DataType, ErrorType>> parseResponse(Future<QueryResult> futureResponse, DataTypeQueryResponseConverter<DataType> dataTypeResponseConverter) async{
    return genericParseResponse(
      futureResponse,
      dataType,
      dataTypeResponseConverter ?? queryResponseConverter?.toDataResponseConverter
    );
  }

  Future<QueryResponse<List<DataType>, ErrorType>> parseResponseAsList(Future<QueryResult> futureResponse, DataTypeQueryResponseConverter<List<DataType>> dataTypeResponseConverter) async{
    return genericParseResponseAsList(
      futureResponse,
      dataType,
      dataTypeResponseConverter ?? queryResponseConverter?.toDataListResponseConverter
    );
  }

  Future<QueryResponse<DataType, ErrorType>> genericParseResponse<DataType>(Future<QueryResult> futureResponse, DataType dataType, DataTypeQueryResponseConverter<DataType> dataTypeResponseConverter) async {
    QueryResult queryResult = await getSaveResponse(futureResponse);
    QueryResponse<DataType, ErrorType> response;
    try{
      dataTypeResponseConverter ??= QueryResponseConverter<DataType>(data: dataType).toDataResponseConverter;
      response = QueryResponse.fromMapQueryResult<DataType, ErrorType>(queryResult, errorType, dataTypeResponseConverter);
    }catch(e){
      response = QueryResponse<DataType, ErrorType>(
        data: null,
        errorType: errorType,
        exception: queryResult?.exception,
        source: queryResult?.source,
        timestamp: queryResult.timestamp,
      );
      print(e);
    }
    return response;
  }

  Future<QueryResponse<List<DataType>, ErrorType>> genericParseResponseAsList<DataType>(Future<QueryResult> futureResponse, DataType dataType, DataTypeQueryResponseConverter<List<DataType>> dataTypeResponseConverter) async {
    QueryResult queryResult = await getSaveResponse(futureResponse);
    QueryResponse<List<DataType>, ErrorType> response;
    try{
      dataTypeResponseConverter ??= QueryResponseConverter<DataType>(data: dataType).toDataListResponseConverter;
      response = QueryResponse.fromListQueryResult<DataType, ErrorType>(queryResult, errorType, dataTypeResponseConverter);
    }catch(e){
      response = QueryResponse<List<DataType>, ErrorType>(
        data: null,
        errorType: errorType,
        exception: queryResult?.exception,
        source: queryResult?.source,
        timestamp: queryResult.timestamp,
      );
      print(e);
    }
    return response;
  }
}
