import 'package:adeptarch/src/api/subscription_response_conveter.dart';
import 'package:graphql_flutter/graphql_flutter.dart';

class SubscriptionResponse<T> {

  int statusCode;
  String reasonPhrase;

  List<dynamic> errors;

  /// List<dynamic> or Map<String, dynamic>
  Map<String, dynamic> extensions;
  Map<String, dynamic> context;

  T data;

  SubscriptionResponse({
    this.data,
    this.statusCode,
    this.reasonPhrase,
    this.errors,
    this.extensions,
    this.context,
  });

  /// Whether the response includes an exception
  bool get hasException => errors != null && errors.isNotEmpty;

  /// Whether the response includes an exception
  bool get isSuccessful => !hasException;

  static SubscriptionResponse<DataType> fromMapQueryResult<DataType>(
      FetchResult fetchResult,
      DataTypeSubscriptionResponseConverter<DataType> dataTypeResponseConverter,){
    DataType data;
    try{
      data = dataTypeResponseConverter(fetchResult);
    }catch(e){
      print(e);
    }
    return SubscriptionResponse<DataType>(
      data: data,
      statusCode: fetchResult?.statusCode,
      reasonPhrase: fetchResult?.reasonPhrase,
      errors: fetchResult?.errors,
      extensions: fetchResult?.extensions,
      context: fetchResult?.context,
    );
  }

  static SubscriptionResponse<List<DataType>> fromListQueryResult<DataType>(
      FetchResult fetchResult,
      DataTypeSubscriptionResponseConverter<List<DataType>> dataTypeResponseConverter){
    List<DataType> data;
    try{
      data = dataTypeResponseConverter(fetchResult);
    }catch(e){
      print(e);
    }
    return SubscriptionResponse<List<DataType>>(
      data: data,
      statusCode: fetchResult?.statusCode,
      reasonPhrase: fetchResult?.reasonPhrase,
      errors: fetchResult?.errors,
      extensions: fetchResult?.extensions,
      context: fetchResult?.context,
    );
  }
}