import 'package:adeptutils/adeptutils.dart';
import 'package:graphql_flutter/graphql_flutter.dart';

typedef DataTypeSubscriptionResponseConverter<T> = T Function(FetchResult fetchResult);

class SubscriptionResponseConverter<T> {
  T data;
  DataTypeSubscriptionResponseConverter<List<T>> toDataListResponseConverter;
  DataTypeSubscriptionResponseConverter<T> toDataResponseConverter;
  SubscriptionResponseConverter({
    this.data,
  }) {
    toDataListResponseConverter = (fetchResult) => toDataList(fetchResult);
    toDataResponseConverter = (fetchResult) => toData(fetchResult);
  }

  List<T> toDataList(FetchResult fetchResult) {
    try{
      if(data == null || fetchResult?.data == null) return null;
      List list;
      if(fetchResult.data is List)
        list = fetchResult.data;
      else
      if(fetchResult.data is Map && (fetchResult.data as Map).isNotEmpty)
        list = (fetchResult.data as Map).values.first;
      if(data is Jsonable)
        return (data as Jsonable).fromJsonList(list);
      if(list.runtimeType == data.runtimeType)
        return list;
    }catch(e){
      print(e);
    }
    return null;
  }

  T toData(FetchResult fetchResult) {
    try{
      if(data == null || fetchResult?.data == null) return null;
      if(data is Jsonable && fetchResult.data is Map && (fetchResult.data as Map).isNotEmpty)
        return (data as Jsonable).fromJsonMap((fetchResult.data as Map).values.first);
      if(fetchResult.data?.runtimeType == data.runtimeType)
        return fetchResult.data;
    }catch(e){
      print(e);
    }
    return null;
  }

}