import 'package:adeptarch/src/filters/filter.dart';
import 'package:adeptutils/adeptutils.dart';
import 'package:json_annotation/json_annotation.dart';

export 'package:adeptarch/src/filters/filter.dart';
export 'package:adeptarch/src/filters/filter_controller.dart';
export 'package:adeptarch/src/utils/sort_mode.dart';
export 'package:adeptutils/src/collections/multi_key_map.dart';

part 'filters.g.dart';

///Run flutter packages pub run build_runner build in order to changes take effect
@JsonSerializable()
class Filters extends Jsonable<Filters>
{
  String key;
  List<TextFilter> textFilters;
  List<NumberFilter> intFilters;
  List<BoolFilter> boolFilters;
  List<DateFilter> dateFilters;
  List<EnumFilter> enumFilters;

  Filters({this.key}) {
    clear();
  }

  Filters clear() {
    textFilters = [];
    intFilters = [];
    boolFilters = [];
    dateFilters = [];
    enumFilters = [];
    return this;
  }

  bool get isEmpty =>
    (textFilters == null || textFilters.isEmpty) &&
    (intFilters == null || intFilters.isEmpty) &&
    (boolFilters == null || boolFilters.isEmpty) &&
    (dateFilters == null || dateFilters.isEmpty) &&
    (enumFilters == null || enumFilters.isEmpty)
  ;

  Filters addFilter(Filter filter){
    if(filter is TextFilter) {textFilters ??= []; textFilters.add(filter);}
    if(filter is NumberFilter) {intFilters ??= []; intFilters.add(filter);}
    if(filter is BoolFilter) {boolFilters ??= []; boolFilters.add(filter);}
    if(filter is DateFilter) {dateFilters ??= []; dateFilters.add(filter);}
    if(filter is EnumFilter) {enumFilters ??= []; enumFilters.add(filter);}
    return this;
  }

  Map<String, dynamic> build() {
    Map<String, dynamic> multiKeyMap = MultiKeyMap();
    textFilters ??= []; textFilters.forEach((filter) => multiKeyMap.addAll(filter.build()));
    intFilters ??= []; intFilters.forEach((filter) => multiKeyMap.addAll(filter.build()));
    boolFilters ??= []; boolFilters.forEach((filter) => multiKeyMap.addAll(filter.build()));
    dateFilters ??= []; dateFilters.forEach((filter) => multiKeyMap.addAll(filter.build()));
    enumFilters ??= []; enumFilters.forEach((filter) => multiKeyMap.addAll(filter.build()));
    return multiKeyMap;
  }

  @override
  Map<String, dynamic> toJson() => _$FiltersToJson(this);
  @override
  Filters fromJsonMap(Map<String, dynamic> json) => Filters.fromJson(json);
  factory Filters.fromJson(Map<String, dynamic> json) => json != null ? _$FiltersFromJson(json) : null;
}