import 'package:adeptarch/src/filters/filter.dart';
import 'package:adeptarch/src/utils/sort_mode.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:flutter/foundation.dart';

part 'number_filter.g.dart';

const int
    NUMBER_FILTER_ALL = 0,
    NUMBER_FILTER_EQUALS = 1,
    NUMBER_FILTER_GREATER_THAN = 2,
    NUMBER_FILTER_GREATER_OR_EQUAL_THAN = 3,
    NUMBER_FILTER_LESS_THAN = 4,
    NUMBER_FILTER_LESS_OR_EQUAL_THAN = 5,
    NUMBER_FILTER_RANGE = 6
;

@JsonSerializable()
class NumberFilter extends Filter<NumberFilter, num>
{
  int numberFilterType;
  num valueMin;
  num valueMax;
  NumberFilter(
    {
      @required String param,
      SortMode sortMode,
      FilterActionType filterActionType,
      this.numberFilterType = NUMBER_FILTER_ALL,
    })
      : super(
      param: param,
      sortMode: sortMode,
      filterActionType: filterActionType,
    );

  @override
  void reset() {
    valueMin = null;
    valueMax = null;
    numberFilterType = DATE_FILTER_ALL;
    super.reset();
  }

  @override
  bool get isReady {
    if(!isValid) return false;
    return true;
  }

  @override
  bool get isValid {
    if(param == null) return false;
    if(numberFilterType == NUMBER_FILTER_RANGE){
      if(valueMin == null || valueMax == null) return false; else
        if(valueMin > valueMax) return false;
    }
    return true;
  }

  Map buildNumberFilter()
  {
    switch(numberFilterType){
      case NUMBER_FILTER_EQUALS:
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.EQUALS, valueMin));
        break;
      case NUMBER_FILTER_GREATER_THAN:
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.GREATER_THAN, valueMin));
        break;
      case NUMBER_FILTER_GREATER_OR_EQUAL_THAN:
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.GREATER_OR_EQUAL_THAN, valueMin));
        break;
      case NUMBER_FILTER_LESS_THAN:
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.LESS_THAN, valueMin));
        break;
      case NUMBER_FILTER_LESS_OR_EQUAL_THAN:
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.LESS_OR_EQUAL_THAN, valueMin));
        break;
      case NUMBER_FILTER_RANGE:
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.GREATER_OR_EQUAL_THAN, valueMin));
        filtersBuilder.putAll(FilterActions.build(filterActionType, param, FilterActions.LESS_OR_EQUAL_THAN, valueMax));
        break;
    }

    return filtersBuilder.build();
  }

  @override
  Map<String, dynamic> build() {
    super.build();
    return buildNumberFilter();
  }

  @override
  Map<String, dynamic> toJson() => _$NumberFilterToJson(this);
  @override
  NumberFilter fromJsonMap(Map<String, dynamic> json) => NumberFilter.fromJson(json);
  factory NumberFilter.fromJson(Map<String, dynamic> json) => json != null ? _$NumberFilterFromJson(json) : null;
}