import 'package:adeptarch/adeptarch.dart';
import 'package:adeptarch/src/dao/super_dao.dart';
import 'package:adeptarch/src/utils/adept_publish_subject.dart';

abstract class SuperSubscriptionRepository< Dao extends SuperDao<DataType>, DataType> 
{
  Dao dao;
  AdeptPublishSubject<SubscriptionResource<DataType>> publishSubject;

  SuperSubscriptionRepository(this.dao) :
    publishSubject = new AdeptPublishSubject()
  ;

  AdeptPublishSubject<SubscriptionResource<DataType>> getObservableData() => publishSubject;

  /// Loading
  void postLoading({DataType resourceData, AdeptPublishSubject<SubscriptionResource<DataType>> publishSubject}) async
  {
    publishSubject ??= this.publishSubject;
    if(resourceData == null) resourceData = publishSubject.value != null ? publishSubject.value.data : null;
    publishSubject.post(SubscriptionResource.loading(resourceData));
  }

  /// Success
  void postData({DataType resourceData, AdeptPublishSubject<SubscriptionResource<DataType>> publishSubject}) async
  {
    publishSubject ??= this.publishSubject;
    if(resourceData == null) resourceData = publishSubject.value != null ? publishSubject.value.data : null;
    publishSubject.post(SubscriptionResource.data(resourceData));
  }

  /// Error
  void postError(String message, {DataType resourceData, int code, dynamic exception, AdeptPublishSubject<SubscriptionResource<DataType>> publishSubject}) async
  {
    publishSubject ??= this.publishSubject;
    if(resourceData == null) resourceData = publishSubject.value != null ? publishSubject.value.data : null;
    publishSubject.post(SubscriptionResource.error(resourceData, message, code: code, exception: exception));
  }

  /// Done
  void postDone({AdeptPublishSubject<SubscriptionResource<DataType>> publishSubject}) async
  {
    publishSubject ??= this.publishSubject;
    publishSubject.post(SubscriptionResource.done());
  }
}