import 'package:adeptarch/src/resource/resource_status.dart';
import 'package:flutter/foundation.dart';

export 'package:adeptarch/src/resource/resource_data_access.dart';
export 'package:adeptarch/src/resource/resource_loader.dart';
export 'package:adeptarch/src/resource/resource_status.dart';

class Resource<T> {

  @required ResourceStatus status;
  String message;
  int code;
  dynamic exception;
  dynamic extras;
  T data;

  Resource({ResourceStatus status = ResourceStatus.ERROR, T data,
    String message, int code, dynamic exception, dynamic extras}) {
    this.status = status;
    this.data = data;
    this.message = message;
    this.code = code;
    this.exception = exception;
    this.extras = extras;
  }

  factory Resource.success(T data, {dynamic extras}) => Resource(status: ResourceStatus.SUCCESS, data: data, extras: extras);
  factory Resource.error(T data, String message, {int code, dynamic exception, dynamic extras}) =>
      Resource(status: ResourceStatus.ERROR, data: data, message: message,
          code: code, exception: exception, extras: extras);
  factory Resource.loading(T data, {dynamic extras}) => Resource(status: ResourceStatus.LOADING, data: data, extras: extras);

  bool exists() => data != null;

  bool equals(Object o) {
    if (this == o) return true;

    if(!(o is Resource)) return false;

    Resource resource = o;
    if (status != resource.status) return false;
    if (code != resource.code) return false;
    if (message != resource.message) return false;
    return data != resource.data;
  }

  static bool hasData(Resource resource) => resource != null && resource.exists();
}
