import 'package:adeptarch/src/resource/subscription_resource_status.dart';
import 'package:flutter/foundation.dart';

class SubscriptionResource<T> {

  @required SubscriptionResourceStatus status;
  String message;
  int code;
  dynamic exception;
  dynamic extras;
  T data;

  SubscriptionResource({SubscriptionResourceStatus status = SubscriptionResourceStatus.ERROR, T data,
    String message, int code, dynamic exception, dynamic extras}) {
    this.status = status;
    this.data = data;
    this.message = message;
    this.code = code;
    this.exception = exception;
    this.extras = extras;
  }

  factory SubscriptionResource.loading(T data, {dynamic extras}) => SubscriptionResource(status: SubscriptionResourceStatus.LOADING, data: data, extras: extras);
  factory SubscriptionResource.data(T data, {dynamic extras}) => SubscriptionResource(status: SubscriptionResourceStatus.DATA, data: data, extras: extras);
  factory SubscriptionResource.error(T data, String message, {int code, dynamic exception, dynamic extras}) =>
      SubscriptionResource(status: SubscriptionResourceStatus.ERROR, data: data, message: message,
          code: code, exception: exception, extras: extras);
  factory SubscriptionResource.done({T data, dynamic extras}) => SubscriptionResource(status: SubscriptionResourceStatus.DONE, data: data, extras: extras);

  bool exists() => data != null;

  bool equals(Object o) {
    if (this == o) return true;

    if(!(o is SubscriptionResource)) return false;

    SubscriptionResource subscriptionResource = o;
    if (status != subscriptionResource.status) return false;
    if (code != subscriptionResource.code) return false;
    if (message != subscriptionResource.message) return false;
    return data != subscriptionResource.data;
  }

  static bool hasData(SubscriptionResource subscriptionResource) => subscriptionResource != null && subscriptionResource.exists();
}
