import 'package:adeptutils/adeptutils.dart';
import 'package:adeptarch/src/filters/filter.dart';

enum SortMode
{
  NONE,
  ASC,
  DESC,
}

class SortModeUtils{
  static const String PARAM_SORT = "sort";

  static const _mapValueOfName = {
    "NONE": SortMode.NONE,
    "ASC": SortMode.ASC,
    "DESC": SortMode.DESC,
  };

  static SortMode valueOf(String name) => _mapValueOfName[name];

  static String nameOf(SortMode value) {
    return value != null ? value.toString().split(".")[1] : null;
  }

  static Map<String, dynamic> build(SortMode sortMode, String param, {FilterActionType filterActionType}){
    if(param == null || param.isEmpty || sortMode == SortMode.NONE) return null;
    switch(filterActionType ?? FilterActionType.FILTER_JPA){
      case FilterActionType.FILTER_SQL:
        return MapBuilder<String, dynamic>(multiKey: true).put(PARAM_SORT, "$param ${nameOf(sortMode)}").build();
      case FilterActionType.FILTER_JPA:
      default:
        return MapBuilder<String, dynamic>(multiKey: true).put(PARAM_SORT, "$param,${nameOf(sortMode)}").build();
    }
  }
}

class SortParam{
  String param;
  SortMode sortMode;

  SortParam(this.param, {SortMode sortMode, String sortModeName}) :
    this.sortMode = sortMode ?? SortModeUtils.valueOf(sortModeName)
  ;

  String get sortModeName => SortModeUtils.nameOf(sortMode);

  static SortParam fromMap(Map<String, dynamic> map, {String defaultParam, SortMode defaultSortMode}){
    SortParam sortParam = SortParam(defaultParam, sortMode: defaultSortMode);
    if(map == null || map.isEmpty) return sortParam;
    SortParam temp = build(map[SortModeUtils.PARAM_SORT]);
    if(temp != null) sortParam = sortParam..
      param = temp.param ?? defaultParam..
      sortMode = temp.sortMode ?? defaultSortMode;
    return sortParam;
  }

  static SortParam build(String data){
    if(data == null) return null;
    SortMode sortMode;
    if(data.toUpperCase().endsWith(SortModeUtils.nameOf(SortMode.ASC)))
      sortMode = SortMode.ASC;
    if(sortMode == null && data.toUpperCase().endsWith(SortModeUtils.nameOf(SortMode.DESC)))
      sortMode = SortMode.DESC;
    if(sortMode == null) return null;
    String param = data.substring(0, (data.length - SortModeUtils.nameOf(sortMode).length) - 1);
    return SortParam(param, sortMode: sortMode);
  }
}