import 'package:adeptarch/src/repository/super_repository.dart';
import 'package:adeptarch/src/utils/adept_publish_subject.dart';
import 'package:adeptarch/src/resource/resource_data_access.dart';
import 'package:adeptarch/src/resource/resource.dart';
import 'package:adeptarch/src/dao/super_dao.dart';
import 'dart:async';


abstract class SuperViewModel<Repository extends SuperRepository<Dao, ResourceData, Data, Param>, Dao extends SuperDao<Data>, ResourceData, Data, Param> implements ResourceDataAccess<ResourceData, Data, Param>
{
  Repository repository;

  SuperViewModel(this.repository);

  AdeptPublishSubject<Resource<ResourceData>> getObservableData() => repository.getObservableData();
  AdeptPublishSubject<Resource<List<ResourceData>>> getObservableDataList() => repository.getObservableDataList();

  @override
  Data loadSync({dynamic params}) => repository?.loadSync(params: params);
  @override
  Future<Data> loadAsync({dynamic params}) => repository?.loadAsync(params: params);
  @override
  AdeptPublishSubject<Resource<ResourceData>> load({List<Param> params, AdeptPublishSubject<Resource<ResourceData>> publishSubject}) => repository.load(params: params, publishSubject: publishSubject);
  @override
  AdeptPublishSubject<Resource<ResourceData>> loadNew({List<Param> params, AdeptPublishSubject<Resource<ResourceData>> publishSubject}) => repository.loadNew(params: params, publishSubject: publishSubject);
  @override
  void save({Data data, List<Data> dataList, bool updateRef, AdeptPublishSubject<Resource<ResourceData>> publishSubject}) => repository.save(data: data, dataList: dataList, updateRef: updateRef, publishSubject: publishSubject);
  @override
  void delete({Data data, List<Data> dataList, bool updateRef, AdeptPublishSubject<Resource<ResourceData>> publishSubject}) => repository.delete(data: data, dataList: dataList, updateRef: updateRef, publishSubject: publishSubject);
  @override
  void clear({bool updateRef, AdeptPublishSubject<Resource<ResourceData>> publishSubject}) => repository.clear(updateRef: updateRef, publishSubject: publishSubject);
  @override
  void notifyChanges({AdeptPublishSubject<Resource<ResourceData>> publishSubject}) => repository.notifyChanges(publishSubject: publishSubject);
}