import 'package:adeptarch/src/widgets/widgets.dart';
import 'package:adeptarch/src/widgets/sort_view.dart';
import 'package:adeptarch/src/filters/filters.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart' as Intl;
import 'dart:ui';
import 'package:flutter/cupertino.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'dart:async';

class DateFilterView extends BaseFilterView<DateFilter> {

  String textDateStart;
  String textDateEnd;
  String textAll;
  String textFromYesterday;
  String textFromToday;
  String textFromNow;
  String textFromTomorrow;
  String textBeforeToday;
  String textAfterToday;
  String textInARange;

  DateTimePickerMode datePickerMode;

  DateFilterView({
    FilterController<DateFilter> filterController,
    String hint,
    bool allowSorting,
    this.textDateStart,
    this.textDateEnd,
    this.textAll,
    this.textFromYesterday,
    this.textFromToday,
    this.textFromNow,
    this.textFromTomorrow,
    this.textBeforeToday,
    this.textAfterToday,
    this.textInARange,
    this.datePickerMode,
  }) : super(
    filterController: filterController,
    hint: hint,
    allowSorting: allowSorting,
  ){
    this.textDateStart ??= "Date start";
    this.textDateEnd ??= "Date end";
    this.textAll ??= "All";
    this.textFromYesterday ??= "From yesterday";
    this.textFromToday ??= "From today";
    this.textFromNow ??= "From now";
    this.textFromTomorrow ??= "From tomorrow";
    this.textBeforeToday ??= "Before today";
    this.textAfterToday ??= "After today";
    this.textInARange ??= "In a range";
    this.datePickerMode ??= DateTimePickerMode.BOTH;
  }

  @override
  State createState()  => _DateFilterViewState();
}

class _DateFilterViewState extends State<DateFilterView> {
  Color colorPrimary, colorError, colorTextTertiary;
  final startController = ValueNotifier<DateTime>(null);
  final endController = ValueNotifier<DateTime>(null);
  List<DropdownMenuItem<int>> dateFilterTypes;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    generateDateFilterTypes();
    onFilterChanged();
  }

  void generateDateFilterTypes(){
    dateFilterTypes = [
      DropdownMenuItem(
        value: DATE_FILTER_ALL,
        child: Text(widget.textAll),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_YESTERDAY,
        child: Text(widget.textFromYesterday),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_TODAY,
        child: Text(widget.textFromToday),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_NOW,
        child: Text(widget.textFromNow),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_TOMORROW,
        child: Text(widget.textFromTomorrow),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_BEFORE_TODAY,
        child: Text(widget.textBeforeToday),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_AFTER_TODAY,
        child: Text(widget.textAfterToday),
      ),
      DropdownMenuItem(
        value: DATE_FILTER_RANGE,
        child: Text(widget.textInARange),
      ),
    ];
  }

  void listeners() {
    widget.filterController.addListener(onFilterChanged);
  }

  void onStartChanged(DateTime dateTime){
    widget.filterController.value.setDateStart(dateTime: dateTime);
  }

  void onEndChanged(DateTime dateTime){
    widget.filterController.value.setDateEnd(dateTime: dateTime);
    if(widget.filterController.hasError && widget.filterController.value.isValid){
      widget.filterController.clearError();
      setState(() {});
    }
  }

  void onFilterChanged(){
    DateFilter filter = widget.filterController.value;
    if(filter != null){
      if(filter.dateStart != null)
        startController.value = filter.dateStart.getDateAsDateTime();
      if(filter.dateEnd != null)
        endController.value = filter.dateEnd.getDateAsDateTime();
    }
  }

  void checkColors(context){
    colorPrimary ??= Theme.of(context).primaryColor;
    colorError ??= Theme.of(context).errorColor;
    colorTextTertiary ??= Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return IntrinsicHeight(
      child: Container(
        child: Row(
          children: <Widget>[
            Expanded(
              child: Container(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    Text(
                      widget.hint,
                      style: TextStyle(color: widget.filterController.value.dateFilterType != DATE_FILTER_ALL ? colorPrimary : colorTextTertiary, fontSize: 12),
                    ),
                    Theme(
                      data: ThemeData(brightness: Theme.of(context).brightness ?? Brightness.light),
                      child: DropdownButton<int>(
                        isExpanded: true,
                        items: dateFilterTypes,
                        value: widget.filterController.value.dateFilterType,
                        onChanged: (dateFilterType){
                          widget.filterController.value.dateFilterType = dateFilterType;
                          widget.filterController.clearError();
                          widget.filterController.value.dateStart = null;
                          widget.filterController.value.dateEnd = null;
                          startController.value = null;
                          endController.value = null;
                          setState(() {});
                        },
                      ),
                    ),
                    widget.filterController.error == null ? Container() :
                    Container(
                      child: Text(
                        widget.filterController.error,
                        style: TextStyle(color: colorError, fontSize: 12),
                      ),
                    ),
                    widget.filterController.value.dateFilterType != DATE_FILTER_RANGE ? Container() :
                    Container(
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          DateTextField(
                            controller: startController,
                            dateTimePickerMode: widget.datePickerMode,
                            onDateTimeSelected: (dateTime) => onStartChanged(dateTime),
                            textInputAction: TextInputAction.done,
                            decoration: InputDecoration(
                              labelText: widget.textDateStart,
                            ),
                          ),
                          DateTextField(
                            controller: endController,
                            dateTimePickerMode: widget.datePickerMode,
                            onDateTimeSelected: (dateTime) => onEndChanged(dateTime),
                            textInputAction: TextInputAction.done,
                            decoration: InputDecoration(
                              labelText: widget.textDateEnd,
                            ),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
            Visibility(
              child: Container(
                margin: EdgeInsets.only(top: 16),
                alignment: Alignment.topCenter,
                child: SortView(
                  sortMode: widget.filterController.value.sortMode,
                  listener: (oldMode, newMode){
                    widget.filterController.value.sortMode = newMode;
                  },
                ),
              ),
              visible: widget.allowSorting,
            ),
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    widget.filterController.removeListener(onFilterChanged);
    super.dispose();
  }
}

enum DateTimePickerMode{
  DATE,
  TIME,
  BOTH,
}

typedef OnDateTimeSelected(DateTime dateTime);

class DateTextField extends StatefulWidget {
  static const String DATE_FORMAT = "EEE, d MMM yyyy hh:mm a";

  Key key;
  ValueNotifier<DateTime> controller;
  FocusNode focusNode;
  InputDecoration decoration;
  TextInputType keyboardType;
  TextInputAction textInputAction;
  TextCapitalization textCapitalization;
  TextStyle style;
  TextAlign textAlign;
  TextDirection textDirection;
  bool autofocus;
  bool obscureText;
  bool autocorrect;
  int maxLines;
  int maxLength;
  bool maxLengthEnforced;
  ValueChanged<String> onChanged;
  VoidCallback onEditingComplete;
  ValueChanged<String> onSubmitted;
  List<TextInputFormatter> inputFormatters;
  bool enabled;
  double cursorWidth;
  Radius cursorRadius;
  Color cursorColor;
  Brightness keyboardAppearance;
  EdgeInsets scrollPadding;
  bool enableInteractiveSelection;
  GestureTapCallback onTap;
  VoidCallback clearButtonAction;
  Color clearButtonColor;
  IconData clearButtonIcon;

  DateTimePickerMode dateTimePickerMode;
  String dateFormat;
  DateTime defaultDate;
  OnDateTimeSelected onDateTimeSelected;

  TextEditingController textController;

  DateTextField({
    this.key,
    this.controller,
    this.focusNode,
    this.decoration = const InputDecoration(),
    this.keyboardType,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.textAlign = TextAlign.start,
    this.textDirection,
    this.autofocus = false,
    this.obscureText = false,
    this.autocorrect = true,
    this.maxLines = 1,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.inputFormatters,
    this.enabled,
    this.cursorWidth = 2.0,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.enableInteractiveSelection = true,
    this.onTap,
    this.clearButtonAction,
    this.clearButtonColor,
    this.clearButtonIcon,
    this.dateTimePickerMode,
    this.dateFormat,
    this.defaultDate,
    this.onDateTimeSelected,
  }) {
    enabled ??= true;
    dateFormat ??= DATE_FORMAT;
    textController ??= TextEditingController();
    controller ??= ValueNotifier(defaultDate);
    if(controller.value != null)
      textController.text = Intl.DateFormat(dateFormat, window.locale.toString()).format(controller.value);
    dateTimePickerMode ??= DateTimePickerMode.BOTH;
  }

  @override
  _DateTextFieldState createState() => _DateTextFieldState();
}

class _DateTextFieldState extends State<DateTextField>{

  bool hasText = false;
  Color onTextColor, onClearTextColor;

  _DateTextFieldState();

  @override
  void initState() {
    super.initState();
    hasText = controllerHasText;
    widget.controller.addListener(onDateChanged);
    widget.textController.addListener(onTextChanged);
    if(widget.focusNode != null)
      widget.focusNode.addListener((){
        if(widget.focusNode.hasFocus && widget.enabled) showPicker();
      });
  }

  void updateDecoration(context){
    if(onTextColor == null) onTextColor = Theme.of(context).primaryColor;
    if(onClearTextColor == null) onClearTextColor = Theme.of(context).disabledColor;

    if(widget.decoration == null) widget.decoration = InputDecoration();

    widget.decoration = widget.decoration.copyWith(
      labelStyle: widget.decoration.labelStyle != null ?
      widget.decoration.labelStyle.copyWith(
          color: hasText ? onTextColor : onClearTextColor
      ) :
      TextStyle(
          color: hasText ? onTextColor : onClearTextColor
      ),
      disabledBorder: widget.decoration.disabledBorder != null ?
      widget.decoration.disabledBorder.copyWith(
          borderSide: BorderSide(
            color: hasText ? onTextColor : onClearTextColor,
            width: hasText ? 2 : 1,
          )
      ) :
      UnderlineInputBorder(
          borderSide: BorderSide(
            color: hasText ? onTextColor : onClearTextColor,
            width: hasText ? 2 : 1,
          )
      ),
    );
  }

  bool get controllerHasText => widget.textController.text != null && widget.textController.text.isNotEmpty;

  void onDateChanged(){
    DateTime dateTime = widget.controller.value;
    if(dateTime != null)
      widget.textController.text = Intl.DateFormat(widget.dateFormat, window.locale.toString()).format(dateTime);
  }

  void notifyOnDateTimeSelected(DateTime dateTime){
    widget.controller.value = dateTime;
    if(widget.onDateTimeSelected != null) widget.onDateTimeSelected(dateTime);
  }

  void onTextChanged(){
    bool controllerHasText = this.controllerHasText;
    if(controllerHasText != hasText){
      hasText = controllerHasText;
      if(!hasText) notifyOnDateTimeSelected(null);
      setState(() {});
    }
  }

  Icon getCustomButtonIcon(BuildContext context) {
    if(widget.clearButtonColor == null) widget.clearButtonColor = Theme.of(context).primaryColor;
    if(widget.clearButtonIcon != null) return Icon(widget.clearButtonIcon, color: widget.clearButtonColor,);
    return Icon(Icons.clear, color: widget.clearButtonColor,);
  }

  Future<DateTime> get _showDatePicker async => await showDatePicker(
    context: context,
    initialDate: widget.controller.value != null ? widget.controller.value : DateTime.now(),
    firstDate: DateTime(1900),
    lastDate: DateTime(2100),
  );

  Future<TimeOfDay> get _showTimePicker async => await showTimePicker(
    context: context,
    initialTime: widget.controller.value != null ?
    TimeOfDay(
        hour: widget.controller.value.hour,
        minute: widget.controller.value.minute
    ) : TimeOfDay.now(),
  );

  void showPicker() async{
    DateTime dateTime;
    switch(widget.dateTimePickerMode){
      case DateTimePickerMode.BOTH:
        dateTime = await _showDatePicker;
        if(dateTime == null) return;
        TimeOfDay time = await _showTimePicker;
        if(time == null) return;
        dateTime = DateTime(
          dateTime.year,
          dateTime.month,
          dateTime.day,
          time.hour,
          time.minute,
        );
        break;
      case DateTimePickerMode.DATE:
        dateTime = await _showDatePicker;
        if(dateTime == null) return;
        dateTime = DateTime(
          dateTime.year,
          dateTime.month,
          dateTime.day,
        );
        break;
      case DateTimePickerMode.TIME:
        TimeOfDay time = await _showTimePicker;
        if(time == null) return;
        dateTime = DateTime.now();
        dateTime = DateTime(
          dateTime.year,
          dateTime.month,
          dateTime.day,
          time.hour,
          time.minute,
        );
        break;
    }
    notifyOnDateTimeSelected(dateTime);
  }

  @override
  Widget build(BuildContext context) {
    updateDecoration(context);
    return Stack(
      children: <Widget>[
        InkWell(
          child: TextField(
            key: widget.key,
            controller: widget.textController,
            focusNode: widget.focusNode,
            decoration: widget.decoration,
            keyboardType: widget.keyboardType,
            textInputAction: widget.textInputAction,
            textCapitalization: widget.textCapitalization,
            style: widget.style,
            textAlign: widget.textAlign,
            textDirection: widget.textDirection,
            autofocus : widget.autofocus ,
            obscureText : widget.obscureText ,
            autocorrect : widget.autocorrect ,
            maxLines: widget.maxLines,
            maxLength: widget.maxLength,
            maxLengthEnforced : widget.maxLengthEnforced ,
            onChanged: widget.onChanged,
            onEditingComplete: widget.onEditingComplete,
            onSubmitted: widget.onSubmitted,
            inputFormatters: widget.inputFormatters,
            enabled: false,
            cursorWidth: widget.cursorWidth,
            cursorRadius: widget.cursorRadius,
            cursorColor: widget.cursorColor,
            keyboardAppearance: widget.keyboardAppearance,
            scrollPadding: widget.scrollPadding,
            enableInteractiveSelection: widget.enableInteractiveSelection,
            onTap: !widget.enabled ? null : widget.onTap,
          ),
          onTap: !widget.enabled ? null : showPicker,
        ),
        Positioned(
          bottom: 0,
          right: 0,
          child: !hasText ? Container() : IconButton(
            icon: getCustomButtonIcon(context),
            onPressed: (){
              if (widget.clearButtonAction != null)
                widget.clearButtonAction();
              else
                widget.textController.clear();
            },
          ),
        ),
      ],
    );
  }

  @override
  void dispose() {
    widget.controller.removeListener(onDateChanged);
    widget.textController.removeListener(onTextChanged);
    if(widget.focusNode != null) widget.focusNode.dispose();
    super.dispose();
  }
}