import 'package:adeptarch/src/widgets/widgets.dart';
import 'package:adeptarch/src/filters/filters.dart';
import 'package:flutter/material.dart';

class EnumFilterView extends BaseFilterView<EnumFilter> {

  bool useTextUnspecified;
  String textUnspecified;
  List<String> entries;
  List<String> entryValues;

  EnumFilterView({
    FilterController<EnumFilter> filterController,
    String hint,
    bool allowSorting,
    this.useTextUnspecified = true,
    this.textUnspecified,
    this.entries,
    this.entryValues,
  }) : super(
    filterController: filterController,
    hint: hint,
    allowSorting: allowSorting,
  ) {
    assert(entries != null);
    assert(entryValues != null);
    assert(entries.length == entryValues.length);
    textUnspecified ??= "Unspecified";
    if(useTextUnspecified){
      entries.insert(0, textUnspecified);
      entryValues.insert(0, null);
    }
  }

  @override
  State createState()  => _EnumFilterViewState();
}

class _EnumFilterViewState extends State<EnumFilterView>{
  Color colorPrimary, colorTextTertiary;
  List<DropdownMenuItem<String>> enumFilterValues;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    generateFilterItems();
    onFilterChanged();
  }

  void generateFilterItems(){
    enumFilterValues = [];
    for(int i = 0; i < widget.entries.length; ++i){
      enumFilterValues.add(
        DropdownMenuItem(
          value: widget.entryValues[i],
          child: Text(widget.entries[i]),
        )
      );
    }
  }

  void listeners() {
    widget.filterController.addListener(onFilterChanged);
  }

  void onFilterChanged(){
    EnumFilter filter = widget.filterController.value;
    if(filter != null){
    }
  }

  void checkColors(context){
    colorPrimary ??= Theme.of(context).primaryColor;
    colorTextTertiary ??= Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return Container(
      child: Row(
        children: <Widget>[
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Text(
                  widget.hint,
                  style: TextStyle(color: widget.filterController.value.value != null ? colorPrimary : colorTextTertiary, fontSize: 12),
                ),
                Theme(
                  data: ThemeData(brightness: Theme.of(context).brightness ?? Brightness.light),
                  child: DropdownButton<String>(
                    isExpanded: true,
                    items: enumFilterValues,
                    value: widget.filterController.value.value,
                    onChanged: (value){
                      if(value == null) {
                        widget.filterController.reset();
                      } else{
                        widget.filterController.value.equals(value);
                        widget.filterController.clearError();
                      }
                      setState(() {});
                    },
                  ),
                ),
              ],
            )
          ),
          Visibility(
            child: Container(
              margin: EdgeInsets.only(top: 16),
              alignment: Alignment.bottomCenter,
              child: SortView(
                sortMode: widget.filterController.value.sortMode,
                listener: (oldMode, newMode){
                  widget.filterController.value.sortMode = newMode;
                },
              ),
            ),
            visible: widget.allowSorting,
          )
        ],
      ),
    );
  }

  @override
  void dispose() {
    widget.filterController.removeListener(onFilterChanged);
    super.dispose();
  }
}