import 'package:adeptarch/src/filters/filters.dart';
import 'package:flutter/material.dart';

typedef OnSortModeChangeListener(SortMode oldMode, SortMode newMode);

class SortView extends StatefulWidget {

  SortMode sortMode;
  Color activeColor, inactiveColor;
  OnSortModeChangeListener listener;
  IconData icon;
  double iconSize;

  SortView({
    this.sortMode = SortMode.NONE,
    this.activeColor,
    this.inactiveColor,
    this.icon,
    this.iconSize,
    this.listener
  }) {
    sortMode ??= SortMode.NONE;
    icon ??= Icons.filter_list;
    iconSize ??= 32;
  }

  @override
  State createState()  => _SortViewState();
}

class _SortViewState extends State<SortView> {
  final textController = AdeptTextEditingController();
  AnimationController rotationController;


  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
//    rotationController = AnimationController(duration: const Duration(milliseconds: 500), vsync: this);
  }

  void listeners() {
  }

  void onModeChanged(SortMode oldMode, SortMode newMode){
//    rotationController.forward(from: 0.0);
    widget.sortMode = newMode;
    if(widget.listener != null) widget.listener(oldMode, newMode);
    setState(() {});
  }

  Color get getColor {
    switch(widget.sortMode){
      case SortMode.ASC:
      case SortMode.DESC:
        return widget.activeColor;
      case SortMode.NONE:
      default:
        return widget.inactiveColor;
    }
  }

  int get getRotation {
    switch(widget.sortMode){
      case SortMode.DESC:
        return 0;
      case SortMode.ASC:
      case SortMode.NONE:
      default:
        return 2;
    }
  }

  SortMode get nextMode {
    switch(widget.sortMode){
      case SortMode.ASC:
        return SortMode.DESC;
      case SortMode.DESC:
        return SortMode.NONE;
      case SortMode.NONE:
      default:
        return SortMode.ASC;
    }
  }

  void checkColors(context){
    widget.activeColor ??= Theme.of(context).accentColor;
    widget.inactiveColor ??= Colors.grey;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return IconButton(
      icon: RotatedBox(
        quarterTurns: getRotation,
        child: Icon(widget.icon, size: widget.iconSize, color: getColor,),
      ),
      onPressed: () => onModeChanged(widget.sortMode, nextMode),
    );
  }

}